package xyz.lychee.lagfixer.libs.adventure.text.minimessage;

import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.function.UnaryOperator;
import xyz.lychee.lagfixer.libs.adventure.pointer.Pointered;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.internal.parser.ParsingExceptionImpl;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.internal.parser.Token;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.internal.parser.node.TagPart;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.tag.Tag;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.tag.resolver.ArgumentQueue;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.tag.resolver.TagResolver;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

class ContextImpl implements Context {
   private static final Token[] EMPTY_TOKEN_ARRAY = new Token[0];
   private final boolean strict;
   private final Consumer<String> debugOutput;
   private String message;
   private final MiniMessage miniMessage;
   private final @Nullable Pointered target;
   private final TagResolver tagResolver;
   private final UnaryOperator<String> preProcessor;
   private final UnaryOperator<Component> postProcessor;

   ContextImpl(final boolean strict, final Consumer<String> debugOutput, final String message, final MiniMessage miniMessage, final @Nullable Pointered target, final @Nullable TagResolver extraTags, final @Nullable UnaryOperator<String> preProcessor, final @Nullable UnaryOperator<Component> postProcessor) {
      this.strict = strict;
      this.debugOutput = debugOutput;
      this.message = message;
      this.miniMessage = miniMessage;
      this.target = target;
      this.tagResolver = extraTags == null ? TagResolver.empty() : extraTags;
      this.preProcessor = preProcessor == null ? UnaryOperator.identity() : preProcessor;
      this.postProcessor = postProcessor == null ? UnaryOperator.identity() : postProcessor;
   }

   public boolean strict() {
      return this.strict;
   }

   public Consumer<String> debugOutput() {
      return this.debugOutput;
   }

   public @NotNull String message() {
      return this.message;
   }

   void message(final @NotNull String message) {
      this.message = message;
   }

   public @NotNull TagResolver extraTags() {
      return this.tagResolver;
   }

   public UnaryOperator<Component> postProcessor() {
      return this.postProcessor;
   }

   public UnaryOperator<String> preProcessor() {
      return this.preProcessor;
   }

   public @Nullable Pointered target() {
      return this.target;
   }

   public @NotNull Pointered targetOrThrow() {
      if (this.target == null) {
         throw this.newException("A target is required for this deserialization attempt");
      } else {
         return this.target;
      }
   }

   public <T extends Pointered> @NotNull T targetAsType(final @NotNull Class<T> targetClass) {
      if (((Class)Objects.requireNonNull(targetClass, "targetClass")).isInstance(this.target)) {
         return (Pointered)targetClass.cast(this.target);
      } else {
         throw this.newException("A target with type " + targetClass.getSimpleName() + " is required for this deserialization attempt");
      }
   }

   public @NotNull Component deserialize(final @NotNull String message) {
      return this.miniMessage.deserialize((String)Objects.requireNonNull(message, "message"), this.tagResolver);
   }

   public @NotNull Component deserialize(final @NotNull String message, final @NotNull TagResolver resolver) {
      return this.miniMessage.deserialize((String)Objects.requireNonNull(message, "message"), TagResolver.builder().resolver(this.tagResolver).resolver((TagResolver)Objects.requireNonNull(resolver, "resolver")).build());
   }

   public @NotNull Component deserialize(final @NotNull String message, final @NotNull TagResolver... resolvers) {
      return this.miniMessage.deserialize((String)Objects.requireNonNull(message, "message"), TagResolver.builder().resolver(this.tagResolver).resolvers((TagResolver[])Objects.requireNonNull(resolvers, "resolvers")).build());
   }

   public @NotNull ParsingException newException(final @NotNull String message) {
      return new ParsingExceptionImpl(message, this.message, (Throwable)null, false, EMPTY_TOKEN_ARRAY);
   }

   public @NotNull ParsingException newException(final @NotNull String message, final @NotNull ArgumentQueue tags) {
      return new ParsingExceptionImpl(message, this.message, (Throwable)null, false, tagsToTokens(((ArgumentQueueImpl)tags).args));
   }

   public @NotNull ParsingException newException(final @NotNull String message, final @Nullable Throwable cause, final @NotNull ArgumentQueue tags) {
      return new ParsingExceptionImpl(message, this.message, cause, false, tagsToTokens(((ArgumentQueueImpl)tags).args));
   }

   private static Token[] tagsToTokens(final List<? extends Tag.Argument> tags) {
      Token[] tokens = new Token[tags.size()];
      int i = 0;

      for(int length = tokens.length; i < length; ++i) {
         tokens[i] = ((TagPart)tags.get(i)).token();
      }

      return tokens;
   }
}
